@@ -14,7 +14,8 @@
 #limitations under the License.
 #++
 require 'Dnsruby/select_thread'
-require 'Dnsruby/iana_ports'
+require 'ipaddr'
+#require 'Dnsruby/iana_ports'
 module Dnsruby
   class PacketSender # :nodoc: all
     @@authoritative_cache = Cache.new
@@ -64,6 +65,11 @@ module Dnsruby
     # 
     # Defaults to false
     attr_accessor :use_tcp
+
+    # Use UDP only - don't use TCP
+    # For test/debug purposes only
+    # Defaults to false
+    attr_accessor :no_tcp
     
     # The TSIG record to sign/verify messages with
     attr_reader :tsig
@@ -76,7 +82,7 @@ module Dnsruby
     # The source address to send queries from
     # 
     # Defaults to localhost
-    attr_accessor :src_address
+    attr_reader :src_address
     
     # should the Recursion Desired bit be set on queries?
     # 
@@ -95,6 +101,24 @@ module Dnsruby
     # dnssec defaults to ON
     attr_reader :dnssec
     
+    # Set the source address. If the arg is nil, do nothing
+    def src_address=(arg)
+      if (not arg.nil?)
+        @src_address = arg
+      end
+      # Do some verification to make sure that the source address
+      # is of the same type as the server address
+      if (@server.nil?)
+        return
+      else
+        si = IPAddr.new(@src_address)
+        i = IPAddr.new(@server)
+        if (i.ipv4? and si.ipv6?) or (i.ipv6? and si.ipv4?)
+          raise ArgumentError.new("Invalid address specified (address family does not match)")
+        end
+      end
+    end
+
     #Sets the TSIG to sign outgoing messages with.
     #Pass in either a Dnsruby::RR::TSIG, or a key_name and key (or just a key)
     #Pass in nil to stop tsig signing.
@@ -133,6 +157,7 @@ module Dnsruby
     # * :server
     # * :port
     # * :use_tcp
+    # * :no_tcp
     # * :ignore_truncation
     # * :src_address
     # * :src_port
@@ -147,6 +172,7 @@ module Dnsruby
       @udp_size = Resolver::DefaultUDPSize
       @dnssec = Resolver::DefaultDnssec
       @use_tcp = false
+      @no_tcp = false
       @tsig = nil
       @ignore_truncation = false
       @src_address        = '0.0.0.0'
@@ -174,6 +200,17 @@ module Dnsruby
       #Check server is IP
       @server=Config.resolve_server(@server)
 
+      begin
+        i = IPv4.create(@server)
+        @src_address = '0.0.0.0'
+      rescue Exception
+        begin
+          i = IPv6.create(@server)
+          @src_address = '::'
+        rescue Exception
+          Dnsruby.log.error{"Server is neither IPv4 or IPv6!\n"}
+        end
+      end
       #      ResolverRegister::register_single_resolver(self)
     end
     
@@ -282,6 +319,13 @@ module Dnsruby
       end
       # Otherwise, run the query
       if (udp_packet_size < query_packet.length)
+        if (@no_tcp)
+          # Can't send the message - abort!
+              err=IOError.new("Can't send message - too big for UDP and no_tcp=true")
+              Dnsruby.log.error{"#{err}"}
+              st.push_exception_to_select(client_query_id, client_queue, err, nil)
+              return
+        end
         Dnsruby.log.debug{"Query packet length exceeds max UDP packet size - using TCP"}
         use_tcp = true
       end
@@ -320,7 +364,7 @@ module Dnsruby
               socket = UDPSocket.new()
             else
               ipv6 = @src_address =~ /:/
-              socket = UDPSocket.new(ipv6 ? Socket::AF_INET6 : Socket::AF_INET)
+              socket = UDPSocket.new(ipv6.nil? ? Socket::AF_INET : Socket::AF_INET6)
             end
             socket.bind(@src_address, src_port)
             socket.connect(@server, @port)
@@ -362,10 +406,10 @@ module Dnsruby
         end
         socket.send(query_bytes, 0)
       rescue Exception => e
-        st.push_exception_to_select(client_query_id, client_queue, err, nil)
-        socket.close
         err=IOError.new("Send failed to #{@server}:#{@port} from #{@src_address}:#{src_port}, use_tcp=#{use_tcp}, exception : #{e}")
         Dnsruby.log.error{"#{err}"}
+        st.push_exception_to_select(client_query_id, client_queue, err, nil)
+        socket.close
         return
       end
       
@@ -431,11 +475,13 @@ module Dnsruby
       #continue until we get one.
       if (@src_port[0] == 0)
         candidate = -1
-        # better to construct an array of all the ports we *can* use, and then just pick one at random!
-        candidate = Iana::UNRESERVED_PORTS[rand(Iana::UNRESERVED_PORTS.length())]
-        #        while (!(Resolver.port_in_range(candidate)))
-        #          candidate = (rand(65535-1024) + 1024)
-        #        end
+        #        # better to construct an array of all the ports we *can* use, and then just pick one at random!
+        #        candidate = Iana::UNRESERVED_PORTS[rand(Iana::UNRESERVED_PORTS.length())]
+        #        #        while (!(Resolver.port_in_range(candidate)))
+        #        #          candidate = (rand(65535-1024) + 1024)
+        #        #        end
+        # @TODO@ Should probably construct a bitmap of the IANA ports...
+        candidate = 50000 + (rand(15535)) # pick one over 50000
         return candidate
       end
       pos = rand(@src_port.length)
@@ -445,10 +491,10 @@ module Dnsruby
     def check_response(response, response_bytes, query, client_queue, client_query_id, tcp)
       # @TODO@ Should send_raw avoid this?
       if (!query.send_raw)
-        if (!check_tsig(query, response, response_bytes))
+        sig_value = check_tsig(query, response, response_bytes)
+        if (sig_value != :okay)
           # Should send error back up to Resolver here, and then NOT QUERY AGAIN!!!
-          return TsigError.new
-          #          return false
+          return sig_value
         end
         # Should check that question section is same as question that was sent! RFC 5452
         # If it's not an update...
@@ -467,12 +513,16 @@ module Dnsruby
         end
       end
       if (response.header.tc && !tcp && !@ignore_truncation)
-        # Try to resend over tcp
-        Dnsruby.log.debug{"Truncated - resending over TCP"}
-        # @TODO@ Are the query options used correctly here? DNSSEC in particular...
-        #        query.send_raw = true # Make sure that the packet is not messed with.
-        send_async(query, client_queue, client_query_id, true)
-        return false
+        if (@no_tcp)
+          Dnsruby.log.debug{"Truncated response - not resending over TCP as no_tcp==true"}
+        else
+          # Try to resend over tcp
+          Dnsruby.log.debug{"Truncated - resending over TCP"}
+          # @TODO@ Are the query options used correctly here? DNSSEC in particular...
+          #        query.send_raw = true # Make sure that the packet is not messed with.
+          send_async(query, client_queue, client_query_id, true)
+          return false
+        end
       end
       return true
     end
@@ -483,19 +533,21 @@ module Dnsruby
           if !query.tsig.verify(query, response, response_bytes)
             # Discard packet and wait for correctly signed response
             Dnsruby.log.error{"TSIG authentication failed!"}
-            return false
+            return TsigError.new
           end
         else
           # Treated as having format error and discarded (RFC2845, 4.6)
+          # but return a different error code, because some servers fail at
+          # this
           Dnsruby.log.error{"Expecting TSIG signed response, but got unsigned response - discarding"}
-          return false
+          return TsigNotSignedResponseError.new
         end
       elsif (response.tsig)
         # Error - signed response to unsigned query
         Dnsruby.log.error{"Signed response to unsigned query"}
-        return false
+        return TsigError.new
       end
-      return true
+      return :okay
     end
     
     def make_query(name, type = Types::A, klass = Classes::IN, set_cd=@dnssec)
@@ -515,12 +567,12 @@ module Dnsruby
           packet.header.rd=@recurse
         end
 
-        # @TODO@ Only do this if the packet has not been prepared already!
+        # Only do this if the packet has not been prepared already!
         if (@dnssec)
           prepare_for_dnssec(packet)
-        
         elsif ((udp_packet_size > Resolver::DefaultUDPSize) && !use_tcp)
           #      if ((udp_packet_size > Resolver::DefaultUDPSize) && !use_tcp)
+          # @TODO@ What if an existing OPT RR is not big enough? Should we replace it?
           add_opt_rr(packet)
         end
       end
@@ -536,7 +588,10 @@ module Dnsruby
       # RFC 3225
       optrr = RR::OPT.new(udp_packet_size)
 
-      packet.add_additional(optrr)
+      # Only one OPT RR allowed per packet - do we already have one?
+      if (packet.additional.rrset(packet.question()[0].qname, Types::OPT).rrs.length == 0)
+        packet.add_additional(optrr)
+      end
     end
 
     def prepare_for_dnssec(packet)
