@@ -142,6 +142,12 @@ module Dnsruby
       
       def calculate_mac(algorithm, data)
         mac=nil
+#+ if (key_size > max_digest_len) {
+#+   EVP_DigestInit(&ectx, digester);
+#+   EVP_DigestUpdate(&ectx, (const void*) key_bytes, key_size);
+#+   EVP_DigestFinal(&ectx, key_bytes, NULL);
+#+   key_size = max_digest_len;
+#+ }
         key = @key.gsub(" ", "")
  #       key = Base64::decode64(key)
         key = key.unpack("m*")[0]
@@ -544,7 +550,7 @@ module Dnsruby
         if (@algorithm!=nil)
           error = @error
           error = "UNDEFINED" unless error!=nil
-          rdatastr = "#{@original_id} #{@time_signed} #{@algorithm}. #{error}";
+          rdatastr = "#{@original_id} #{@time_signed} #{@algorithm.to_s(true)} #{error}";
           if (@other_size > 0 && @other_data!=nil)
             rdatastr += " #{@other_data}"
           end
